@echo off & setlocal enableextensions enabledelayedexpansion

if not [%cmdbox%]==[true] (
  echo To be run only in Timo's customized command box
  pause
  goto :EOF)

echo Ŀ
echo  MAILDATE.CMD Build a message's "From" header  
echo  By Prof. Timo Salmi, http://www.uwasa.fi/~ts/ 
echo  Last modified Sun 9-Oct-2005                  
echo 
echo.

:: Help
set wanthelp_=
if [%1]==[?] set wanthelp_=true
if [%1]==[/?] set wanthelp_=true
if /i [%1]==[/help] set wanthelp_=true
if defined wanthelp_ (
  echo This script forms the beginning "From" line of an email message
  echo header. It is based on the "From: " and the "Date: " header fields.
  echo The information can be needed since sometimes the beginning "From"
  echo header is broken or missing altogether. The script is not perfect,
  echo but it should be able to handle the most common situations.
  echo The SED.EXE third party program is needed to use this script.
  echo.
  echo Usage: MAILDATE [EmailOrNewsFileName]
  echo        The default FileName is C:\_M\mail.txt
  goto :EOF)

:: Test for the availability of SED.EXE in the current folder or at path
set found_=
for %%f in ("SED.EXE") do set found_="%%~$PATH:f"
if exist SED.EXE set found_="SED.EXE"
if [%found_%]==[""] (
  echo SED.EXE needed by the script not found at path
  echo.
  echo If you do not have it, get it for example from
  echo   20368 Oct 1 1991 ftp://garbo.uwasa.fi/pc/unix/sed15x.zip
  echo   HHSED executable, E.Raymond+D.Kirschbaum+H.Helman
  echo  or
  echo   878915 Oct 25 2003 ftp://garbo.uwasa.fi/win95/unix/UnxUpdates.zip
  echo   UnxUpdates.zip Updates for UnxUtils GNU utilities for native Win32
  goto :EOF)

:: Get the source filename. You can customize the default to fit yours.
if [%1]==[] (
  set source_=C:\_M\mail.txt
  ) else (
  set source_=%~1)

:: Test for source existence
if not exist "%source_%" (
  echo Exiting: File "%source_%" not found
  echo.
  echo For usage: MAILDATE /?
  goto :EOF)

:: Get the first "Date: " line
set s_=
set count_=
for /f "tokens=* delims=" %%a in (
  'findstr /b /c:"Date: " "%source_%"') do (
  set s1_=%%a
  set /a count_+=1
  if !count_! EQU 1 set s_=!s1_!)

:: Was a "Date: " line found in the email?
if not defined s_ (
  echo No "Date: " line found in "%source_%"
  goto :EOF)

:: Display the fields
findstr /b /c:"From: " "%source_%"|sed -n 1p
echo %s_%
echo.

:: Parse the "Date: " line
for /f "tokens=1-6 delims=, " %%a in ('echo "%s_%"') do (
  set wd_=%%b
  set mm_=%%d
  set dd_=%%c
  set tt_=%%f
  set yy_=%%e
  )

:: Process the day number
if [%dd_:~0,1%]==[0] set dd_=%dd_:~1,1%
set d1_=%dd_:~1,1%
if not defined d1_ set dd_= %dd_%

:: Get the first "From: " line
set s_=
set count_=
for /f "tokens=* delims=" %%a in (
  'findstr /b /c:"From: " "%source_%"^|sed -e "s/</:/g" -e "s/>//g"') do (
  set s1_=%%a
  set /a count_+=1
  if !count_! EQU 1 set s_=!s1_!)

:: Test for parentheses
set parenth_=
echo %s_%|find "(">nul
if %errorlevel% EQU 0 set parenth_=true

:: Parse the "From: " line
set email_=
if defined parenth_ (
  for /f "tokens=1-3 delims=:() " %%a in ('echo "%s_%"') do (
    set email_=%%b)
    ) else (
  for /f "tokens=1-3 delims=:" %%a in ('echo "%s_%"') do (
    set email_=%%c
    if not defined email_ set email_=%%b
    )
  )

:: Remove potential quotes
if defined email_ set email_=%email_:"=%
if defined email_ set email_=%email_: =%
if defined wd_ set wd_=%wd_:"=%
if defined mm_ set mm_=%mm_:"=%
if defined dd_ set dd_=%dd_:"=%
if defined tt_ set tt_=%tt_:"=%
if defined yy_ set yy_=%yy_:"=%

:: Test for the weekday lest it is missing in the original message
set wdok_=
for %%w in (Mon Tue Wed Thu Fri Sat Sun) do (
  if /i [%wd_%]==[%%w] set wdok_=true
  )
if not defined wdok_ (
  echo Warning: The original message's Date: header is deficient.
  echo The format should be for example Date: Sun, 09 Oct 2005 09:28:29
  echo The weekday is missing, or %wd_% is otherwise faulty.
  echo The constructed From header below will be incorrect.
  echo.
  )

:: Test for the month
set mmok_=
for %%w in (Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec) do (
  if /i [%mm_%]==[%%w] set mmok_=true
  )
if defined wdok_ if not defined mmok_ (
  echo Warning: The original message's Date: header is deficient.
  echo The month field %mm_% is faulty or misplaced.
  echo The constructed From header below is likely to be incorrect.
  echo.
  )

:: Check for the email address
echo. %email_%|find "@">nul
if %errorlevel% GTR 0 (
  echo Warning: Was not able to extract a valid email address.
  echo The expected address formats for the original message are
  echo   Timo Salmi ^<ts@uwasa.fi^>
  echo   ts@uwasa.fi ^(Timo Salmi^)
  echo   ts@uwasa.fi
  echo.
)

:: Display the result
echo From %email_%  %wd_% %mm_% %dd_% %tt_% %yy_%

endlocal & goto :EOF
